'''
Created on 22-04-2013

@author: Damian Rusinek <damian.rusinek@gmail.com>
'''

from aqopa.model.parser.lex_yacc import LexYaccParserExtension
from aqopa.model import AssignmentInstruction,\
    CommunicationInstruction,\
    WhileInstruction, IfInstruction, ContinueInstruction, FinishInstruction,\
    CallFunctionInstruction, COMMUNICATION_TYPE_OUT, COMMUNICATION_TYPE_IN,\
    HostSubprocess


class Builder():

    def build_subprocess(self, token):
        """
        host_subprocess : SUBPROCESS IDENTIFIER host_channels BLOCKOPEN instructions_list BLOCKCLOSE
                    | SUBPROCESS IDENTIFIER host_channels BLOCKOPEN instructions_list BLOCKCLOSE SEMICOLON
        """
        s = HostSubprocess(token[2], token[5])
        
        all_channels_active = '*' in token[3]
        if all_channels_active:
            s.all_channels_active = True
        else:
            s.active_channels = token[3]
        
        return s

class ModelParserExtension(LexYaccParserExtension):
    """
    Extension for parsing functions
    """
    
    def __init__(self):
        LexYaccParserExtension.__init__(self)

        self.open_blocks_cnt_by_state = {}
        
        self.builder = Builder()
        
    ##########################################
    #           RESERVED WORDS
    ##########################################
    
    def word_subprocess_specification(self, t):
        t.lexer.push_state('subprocess')
        return t
    
    ##########################################
    #                TOKENS
    ##########################################
    
    def token_block_open(self, t):
        r'{'
        state = t.lexer.current_state()
        state += str(t.lexer.lexstatestack.count(state))
        
        if state not in self.open_blocks_cnt_by_state:
            self.open_blocks_cnt_by_state[state] = 0 
        self.open_blocks_cnt_by_state[state] += 1
        return t
    
    def token_block_close(self, t):
        r'}'
        state = t.lexer.current_state()
        state += str(t.lexer.lexstatestack.count(state))
        
        self.open_blocks_cnt_by_state[state] -= 1
        if self.open_blocks_cnt_by_state[state] == 0:
            t.lexer.pop_state()
        return t
    
    def token_host_rparan(self, t):
        r'\)'
        t.lexer.push_state('hostrparan')
        return t

    ##########################################
    #                RULES
    ##########################################
    
    def instructions_list(self, t):
        """
        instructions_list : instruction
                        | instructions_list instruction
        """
        if len(t) == 2:
            t[0] = []
            t[0].append(t[1])
        else:
            t[0] = t[1]
            t[0].append(t[2])
    
    def instruction(self, t):
        """
        instruction : instruction_assignment
                | instruction_subprocess
                | instruction_communication
                | instruction_while
                | instruction_if
                | instruction_special_command
                | instruction_call_function
        """
        t[0] = t[1]
    
    def instruction_subprocess(self, t):
        """
        instruction_subprocess : SUBPROCESS IDENTIFIER host_channels BLOCKOPEN instructions_list BLOCKCLOSE
                    | SUBPROCESS IDENTIFIER host_channels BLOCKOPEN instructions_list BLOCKCLOSE SEMICOLON
        """
        t[0] = self.builder.build_subprocess(t)
    
    def instruction_assignment(self, t):
        """
        instruction_assignment : IDENTIFIER EQUAL expression_simple SEMICOLON
        """
        t[0] = AssignmentInstruction(t[1], t[3])
    
    def instruction_communication(self, t):
        """
        instruction_communication : IN LPARAN IDENTIFIER COLON identifiers_list RPARAN SEMICOLON
                                | OUT LPARAN IDENTIFIER COLON identifiers_list RPARAN SEMICOLON
        """
        communication_type = COMMUNICATION_TYPE_OUT
        if t[1].lower() == 'in':
            communication_type = COMMUNICATION_TYPE_IN
            
        t[0] = CommunicationInstruction(communication_type, t[3], t[5])
    
    def instruction_while(self, t):
        """
        instruction_while : WHILE LPARAN expression_conditional RPARAN BLOCKOPEN instructions_list BLOCKCLOSE
                        | WHILE LPARAN expression_conditional RPARAN BLOCKOPEN instructions_list BLOCKCLOSE SEMICOLON
        """
        t[0] = WhileInstruction(t[3], t[6])
    
    def instruction_if(self, t):
        """
        instruction_if : IF LPARAN expression_conditional RPARAN BLOCKOPEN instructions_list BLOCKCLOSE 
                    | IF LPARAN expression_conditional RPARAN BLOCKOPEN instructions_list BLOCKCLOSE SEMICOLON
                    | IF LPARAN expression_conditional RPARAN BLOCKOPEN instructions_list BLOCKCLOSE ELSE BLOCKOPEN instructions_list BLOCKCLOSE
                    | IF LPARAN expression_conditional RPARAN BLOCKOPEN instructions_list BLOCKCLOSE ELSE BLOCKOPEN instructions_list BLOCKCLOSE SEMICOLON
        """
        
        conditional = t[3]
        true_instructions = t[6]
        false_instructions = []
        if len(t) > 10:
            false_instructions = t[10]
        
        t[0] = IfInstruction(conditional, true_instructions, false_instructions)
    
    def instruction_special_command(self, t):
        """
        instruction_special_command : CONTINUE SEMICOLON
                                | STOP SEMICOLON
                                | END SEMICOLON
        """
        if t[1].lower() == 'continue':
            t[0] = ContinueInstruction()
        elif t[1].lower() == 'end':
            t[0] = FinishInstruction('end')
        elif t[1].lower() == 'stop':
            t[0] = FinishInstruction('stop')
    
    def instruction_call_function(self, t):
        """
        instruction_call_function : expression_call_function SEMICOLON
        """
        t[0] = CallFunctionInstruction(t[1].function_name, t[1].arguments, t[1].qop_arguments)
    
    
    def _extend(self):
        
        self.parser.add_state('subprocess', 'inclusive')
        self.parser.add_reserved_word('subprocess', 'SUBPROCESS', func=self.word_subprocess_specification, state='process')
           
        self.parser.add_token('BLOCKOPEN', func=self.token_block_open, states=['subprocess'])
        self.parser.add_token('BLOCKCLOSE', func=self.token_block_close, states=['subprocess'])
        self.parser.add_token('RPARAN', func=self.token_host_rparan, states=['subprocess'])
        
        self.parser.add_rule(self.instructions_list)
        self.parser.add_rule(self.instruction)
        self.parser.add_rule(self.instruction_subprocess)
        self.parser.add_rule(self.instruction_assignment)
        self.parser.add_rule(self.instruction_communication)
        self.parser.add_rule(self.instruction_while)
        self.parser.add_rule(self.instruction_if)
        self.parser.add_rule(self.instruction_special_command)
        self.parser.add_rule(self.instruction_call_function)
        

    


'''
Created on 07-09-2013

@author: Damian Rusinek <damian.rusinek@gmail.com>
'''
import sys

from aqopa.simulator.state import Hook

class PrintResultsHook(Hook):

    def __init__(self, module, simulator, output_file=sys.stdout):
        """ """
        self.module = module
        self.simulator = simulator
        self.output_file = output_file
        
    def execute(self, context):
        """ """
        
        self.output_file.write('-'*20)
        self.output_file.write('\n')
        self.output_file.write('Version: %s\n' % self.simulator.context.version.name)
        
        if self.simulator.infinite_loop_occured():
            self.output_file.write('ERROR: Infinite loop on {0} -> {1}\n'.format(
                                unicode(self.simulator.context.get_current_host()),
                                unicode(self.simulator.context.get_current_instruction())))
            self.output_file.write('\n')
            
        for h in context.hosts:
            self.output_file.write('{0: <15}: {1: <15} '.format(h.name, str(self.module.get_current_time(self.simulator, h))))
            if h.finished():
                self.output_file.write('Finished')
                if h.get_finish_error():
                    self.output_file.write('Finished with error: {0}'.format(h.get_finish_error()))
            else:
                self.output_file.write('NOT Finished: {0}'.format(unicode(h.get_current_instructions_context()\
                                                                .get_current_instruction())))
            self.output_file.write("\n")
            
        print ""
        print "Dropped messages:"
        i = 0
        for c in context.channels_manager.channels:
            if c.get_number_of_dropped_messages() > 0:
                i += 1
                print '%s \t %d' % (c.name, c.get_number_of_dropped_messages()) 
        if i == 0:
            print "None"
        self.output_file.write('-'*20)
        self.output_file.write('\n')



'''
Created on 30-10-2013

@author: damian
'''

import optparse
import sys
import os

from aqopa import VERSION
from aqopa.bin import console, gui


def gui_command():
    
    app = gui.AqopaApp(False)
    app.MainLoop()

def console_command():
    parser = optparse.OptionParser()
    parser.usage = "%prog [options]"
    parser.add_option("-f", "--model-file", dest="model_file",  metavar="FILE",
                      help="specifies model file")
    parser.add_option("-m", "--metrics-file", dest="metrics_file",  metavar="FILE",
                      help="specifies file with metrics")
    parser.add_option("-c", "--config-file", dest="config_file",  metavar="FILE",
                      help="specifies file with modules configuration")
    parser.add_option("-s", "--states", dest="save_states", action="store_true", default=False,
                      help="save states flow in a file")
    parser.add_option("-p", '--progressbar', dest="show_progressbar", action="store_true", default=False,
                      help="show the progressbar of the simulation")
    parser.add_option("-V", '--version', dest="show_version", action="store_true", default=False,
                      help="show version of AQoPA")
    parser.add_option("-d", "--debug", dest="debug", action="store_true", default=False,
                      help="DEBUG mode")
    
    (options, args) = parser.parse_args()
    
    if options.show_version:
        print "AQoPA (version %s)" % VERSION
        sys.exit(0)
    
    if not options.model_file:
        parser.error("no qopml model file specified")
    if not os.path.exists(options.model_file):
        parser.error("qopml model file '%s' does not exist" % options.model_file)
        
    if not options.metrics_file:
        parser.error("no metrics file specified")
    if not os.path.exists(options.metrics_file):
        parser.error("metrics file '%s' does not exist" % options.metrics_file)
        
    if not options.config_file:
        parser.error("no configuration file specified")
    if not os.path.exists(options.config_file):
        parser.error("configuration file '%s' does not exist" % options.config_file)
    
    
    f = open(options.model_file, 'r')
    qopml_model = f.read()
    f.close()
    f = open(options.metrics_file, 'r')
    qopml_metrics = f.read()
    f.close()
    f = open(options.config_file, 'r')
    qopml_config = f.read()
    f.close()

    console.run(qopml_model, qopml_metrics, qopml_config, 
         save_states=options.save_states, debug=options.debug,
         show_progressbar=options.show_progressbar)
    


#!/usr/bin/env python
'''
Created on 28-10-2013

@author: Damian Rusinek <damian.rusinek@gmail.com>
'''

from setuptools import setup, find_packages
import aqopa

def read(filename):
    with open(filename) as f:
        return f.read()


setup(name='AQoPA',
      
      version=aqopa.VERSION,
      description='Automated Quality of Protection Analysis Tool for QoP-ML models.',
      long_description=read('README.md'),
      author='Damian Rusinek',
      author_email='damian.rusinek@gmail.com',
      url='http://qopml.org/aqopa/',
      
      license='Freeware',
      platforms=["any"],
      
      install_requires=[read('requirements').split("\n")],
      
      packages=find_packages(),
      package_data={'aqopa': ['bin/assets/logo.png']},
      
      entry_points = {
        'console_scripts': [
            'aqopa-gui = aqopa.cmd:gui_command',
            'aqopa-console = aqopa.cmd:console_command',
        ]
      },
      
      classifiers = [
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Information Technology',
        'Intended Audience :: Science/Research',
        'Intended Audience :: System Administrators',
        'Intended Audience :: Telecommunications Industry',
        'License :: Freeware',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Topic :: Scientific/Engineering :: Information Analysis',
        'Topic :: Security',
        'Topic :: Security :: Cryptography',
        'Topic :: System :: Monitoring',
      ]
     )